Package: aplotExtra
Title: Creating Composite Plots using 'aplot'
Version: 0.0.4
Authors@R: c(
    person(given = "Guangchuang", family = "Yu", email = "guangchuangyu@gmail.com",   role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person("Shuangbin", "Xu", email = "xshuangbin@163.com", role = "ctb", comment = c(ORCID="0000-0003-3513-5362")),
    person("Chun-Hui","Gao", email="gaospecial@gmail.com", role= "ctb", comment=c(ORCID = "0000-0002-1445-7939")),
    person("Shensuo","Li", email="lishensuo@163.com", role= "ctb", comment=c(ORCID = "0009-0006-3229-3700"))
    )
Description: Many complex plots are actually composite plots, such as 'oncoplot', 'funkyheatmap', 'upsetplot', etc. We can produce subplots using 'ggplot2' and combine them to create composite plots using 'aplot'. In this way, it is easy to customize these complex plots, by adding, deleting or modifying subplots in the final plot. This package provides a set of utilities to help users to create subplots and complex plots.
Depends: R (>= 4.1.0)
Imports: aplot (>= 0.2.3), dplyr, forcats, ggfun (>= 0.1.1), ggplot2,
        grid, maftools, purrr, rlang, stats, tibble, tidyr, utils,
        ggstar, yulab.utils (>= 0.0.8)
Suggests: ggtree, data.table, RColorBrewer, R.utils
URL: https://github.com/YuLab-SMU/aplotExtra
License: Artistic-2.0
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-06-12 02:36:09 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [ctb] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Chun-Hui Gao [ctb] (ORCID: <https://orcid.org/0000-0002-1445-7939>),
  Shensuo Li [ctb] (ORCID: <https://orcid.org/0009-0006-3229-3700>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-12 04:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:33:37 UTC; unix

Package: aplotExtra
Title: Creating Composite Plots using 'aplot'
Version: 0.0.6
Authors@R: c(
    person(given = "Guangchuang", family = "Yu", email = "guangchuangyu@gmail.com",   role  = c("aut", "cre"), comment = c(ORCID = "0000-0002-6485-8781")),
    person("Shuangbin", "Xu", email = "xshuangbin@163.com", role = "ctb", comment = c(ORCID="0000-0003-3513-5362")),
    person("Chun-Hui","Gao", email="gaospecial@gmail.com", role= "ctb", comment=c(ORCID = "0000-0002-1445-7939")),
    person("Shensuo","Li", email="lishensuo@163.com", role= "ctb", comment=c(ORCID = "0009-0006-3229-3700"))
    )
Description: Many complex plots are actually composite plots, such as 'oncoplot', 'funkyheatmap', 'upsetplot', etc. We can produce subplots using 'ggplot2' and combine them to create composite plots using 'aplot'. In this way, it is easy to customize these complex plots, by adding, deleting or modifying subplots in the final plot. This package provides a set of utilities to help users to create subplots and complex plots.
Depends: R (>= 4.1.0)
Imports: aplot (>= 0.2.3), dplyr, forcats, ggfun (>= 0.1.1), ggplot2,
        grid, maftools, purrr, rlang, stats, tibble, tidyr, utils,
        ggstar, yulab.utils (>= 0.0.8)
Suggests: ggtree, data.table, RColorBrewer, R.utils, testthat (>=
        3.0.0)
URL: https://github.com/YuLab-SMU/aplotExtra
License: Artistic-2.0
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-04 09:13:57 UTC; HUAWEI
Author: Guangchuang Yu [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6485-8781>),
  Shuangbin Xu [ctb] (ORCID: <https://orcid.org/0000-0003-3513-5362>),
  Chun-Hui Gao [ctb] (ORCID: <https://orcid.org/0000-0002-1445-7939>),
  Shensuo Li [ctb] (ORCID: <https://orcid.org/0009-0006-3229-3700>)
Maintainer: Guangchuang Yu <guangchuangyu@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-04 15:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(funky_bar)
export(funky_heatmap)
export(funky_point)
export(funky_text)
export(get_all_subsets)
export(oncoplot)
export(upset_plot)
import(maftools)
importFrom(aplot,insert_bottom)
importFrom(aplot,insert_right)
importFrom(aplot,insert_top)
importFrom(aplot,plot_list)
importFrom(aplot,xlab2)
importFrom(aplot,ylab2)
importFrom(dplyr,mutate)
importFrom(forcats,as_factor)
importFrom(ggfun,theme_blinds)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotation_custom)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggstar,geom_star)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(purrr,reduce)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,pivot_longer)
importFrom(utils,modifyList)
importFrom(yulab.utils,combinations)
importFrom(yulab.utils,get_fun_from_pkg)

##' @importFrom ggfun theme_blinds
funky_setting <- function(gglist, options) {
  gglist <- lapply(gglist, function(p) p + ggfun::theme_blinds(colour = c('grey90', 'white')))
  if (!is.null(options)) {
      gglist <- lapply(gglist, function(p) p + options)
  }

  return(gglist)
}

#' @importFrom ggplot2 labs
funky_fill_label <- function(data, cols) {
  fill_label <- NULL

  if (length(cols) == 1) {
    label <- names(data)[cols]
    fill_label <- labs(fill = label)
  } 

  return(fill_label)
}

##' @importFrom ggplot2 element_text
##' @importFrom ggplot2 margin
##' @importFrom grid unit
funky_theme <- function() {
    theme(axis.ticks.y = element_blank(),
        axis.text.x = element_text(angle = -30, hjust = 0, size=13),
        axis.text.y = element_blank(),
        axis.line.y = element_blank(),
        axis.title = element_blank(),
        axis.ticks.length.y = unit(0,"pt"),
        plot.margin = margin(0,0,0,0),
        panel.grid.major.x = element_blank(),
        panel.grid.minor.x = element_blank()
    ) 
}

##' @importFrom tibble rownames_to_column
funky_id <- function(data) {
  if (!'id' %in% colnames(data)) {
    data <- tibble::rownames_to_column(data, "id")
  }
  return(data)
}

##' @importFrom dplyr mutate
##' @importFrom tidyr pivot_longer
funky_data <- function(data, cols, cols2 = NULL) {
    data <- funky_id(data)
    data <- data[, c(1, cols, cols2)] |>
      funky_id() 
    cols2 <- setdiff(seq_len(ncol(data)), seq_len(length(cols))+1)
    exclude_names <- colnames(data)[cols2]
    data |>
      tidyr::pivot_longer(-tidyr::all_of(exclude_names)) |>
      dplyr::mutate(id=factor(.data$id, levels = rev(data$id)))
}

##' create text plot (i.e., rownames) for funkyheatmap
##'
##' 
##' @title funky_text
##' @param data data frame
##' @param cols selected column
##' @param hjust text alignment adjustment
##' @return ggplot object
##' @importFrom ggplot2 xlim
##' @importFrom ggplot2 geom_text
##' @importFrom ggplot2 element_blank
##' @importFrom rlang .data
##' @export
##' @author Guangchuang Yu
funky_text <- function(data, cols = 1, hjust=0) {
  d <- funky_data(data, cols)  
  ggplot(d, aes(x=1, y=.data$id, label=.data$id)) + 
    xlim(1,2) +
    geom_text(hjust=hjust) + 
    funky_theme() +
    theme(axis.text.x = element_blank(),
        axis.ticks.x = element_blank()) 
}

##' create dot plot for funkyheatmap
##'
##' 
##' @title funky_point
##' @param data data frame
##' @param cols selected columns
##' @param cols2 selected columns to keep names 
##' @param ... additional parameters, passing to \code{\link[ggstar]{geom_star}}
##' @return ggplot object
##' @importFrom ggplot2 geom_point
##' @importFrom ggstar geom_star
##' @importFrom utils modifyList
##' @export
##' @author Guangchuang Yu
funky_point <- function(data, cols, cols2 = NULL, ...) {
  d2 <- funky_data(data, cols, cols2)

  p <- ggplot(d2, aes(.data$name, .data$id)) #+ 
  mapping <- aes(size = .data$value, fill = .data$value)
  dots <- list(...)
  if (length(dots) >= 1){
    for (i in seq_len(length(dots))){
      if (inherits(dots[[i]], "uneval")){
        mapping <- modifyList(mapping, dots[[i]]) 
        dots[[i]] <- NULL
      }
    }
  }

  if ('stroke' %in% names(dots)){
    names(dots)[names(dots) == "stroke"] <- 'starstroke'
  }else if (!any(names(dots) %in% c('stroke', 'starstroke'))){
    dots[["starstroke"]] <- .3
  }
  if ('shape' %in% names(dots)){
    names(dots)[names(dots) == 'shape'] <- 'starshape'
  }else if ('shape' %in% names(mapping)){
    names(mapping)[names(mapping) == 'shape'] <- 'starshape'
  }else if (!any(names(dots) %in% c('shape', 'starshape')) && !any(names(dots) %in% names(mapping))){
    dots[['starshape']] <- 15
  }
  dots$mapping <- mapping
  point_layers <- do.call('geom_star', dots)
  p <- p + point_layers + funky_theme()
  # p <- p + funky_fill_label(data, cols)
  return(p)
}



##' create bar plot for funkyheatmap
##'
##' 
##' @title funky_bar
##' @param data data frame
##' @param cols selected columns
##' @return ggplot object
##' @importFrom ggplot2 geom_col
##' @importFrom ggplot2 geom_vline
##' @importFrom ggplot2 scale_x_continuous
##' @export
##' @author Guangchuang Yu
funky_bar <- function(data, cols) {
  d2 <- funky_data(data, cols)
  if (length(cols) == 1) {
    label = names(data)[cols]
    mapping <- aes(fill = .data$value)
    position <- 'stack'
  } else {
    label = "name"
    mapping <- aes(fill = .data$name)
    name.levels <- names(data)[cols]
    d2 <- d2 |> dplyr::mutate(name = factor(.data$name, levels = name.levels))
    position <- 'fill'
  }


  p <- ggplot(d2, aes(.data$value, .data$id)) + 
    #geom_col(aes(fill=.data$value), color='black', linewidth=0.3) + 
    geom_col(mapping=mapping, position=position, color='black', linewidth=0.3) +
    funky_theme() +
    #geom_vline(xintercept = 0, linetype="dashed", linewidth=0.8) +
    geom_vline(xintercept = 1, linetype="dashed", linewidth=0.8) 
    #scale_fill_gradient(low = "#CC4C02", high = "#FFFFE5") +

  if (label == "") {
    p <- p + scale_x_continuous(expand=c(0,0)) 
  } else {
    p <- p + scale_x_continuous(breaks = 0.5, labels=label, expand=c(0,0)) 
  }

  # p <- p + funky_fill_label(data, cols)
  return(p)
}


##' create a funkyheatmap
##'
##' 
##' @title funky_heatmap
##' @param ... funky plots (e.g., outputs of `funky_point`, `funky_bar`, etc.)
##' @param data If data is provided, create a funkyheatmap from it. Otherwise, create composite plot from `...`
##' @param widths relative widths of the plots
##' @param options any ggplot component that can be added to the plots 
##' @return gglist object
##' @importFrom aplot plot_list
##' @export
##' @author Guangchuang Yu
funky_heatmap <- function(..., data=NULL, widths = NULL, options=NULL) {

    nrow = 1
    guides = 'collect'
    output = "gglist"
    
    if (!is.null(data)) {
        pg <- list(funky_text(data), 
                funky_point(data, (1:ncol(data))+1)
                )
        if (is.null(widths)) widths <- c(1,3)
    } else {
      pg <- list(...)
    }
    
    if (!is.null(names(pg))) {
      options <- list(theme(plot.margin = margin(r=2)), options)
    }
    
    pg <- funky_setting(pg, options)

    plot_list(gglist=pg, 
        nrow=nrow, 
        widths=widths, 
        guides=guides, 
        output=output
        )
}

# Tests for funkyheatmap.R data/theme functions ---------------------------------

# ---- funky_id ----

test_that("funky_id() adds id column from rownames when id is absent", {
  df <- data.frame(x = 1:3, y = 4:6)
  rownames(df) <- c("row1", "row2", "row3")
  res <- funky_id(df)
  expect_true("id" %in% colnames(res))
  expect_equal(res$id, c("row1", "row2", "row3"))
})

test_that("funky_id() preserves existing id column", {
  df <- data.frame(id = c("a", "b"), x = 1:2)
  res <- funky_id(df)
  expect_equal(res$id, c("a", "b"))
  expect_equal(ncol(res), 2)
})

# ---- funky_data ----

test_that("funky_data() pivots selected all-numeric columns to long format", {
  # After funky_id adds "id" as col 1, original cols shift right by 1
  df <- data.frame(expr1 = c(2.1, 3.5), expr2 = c(1.8, 4.2))
  rownames(df) <- c("BRCA1", "TP53")
  res <- funky_data(df, cols = c(2, 3))  # cols after id addition
  expect_s3_class(res, "data.frame")
  expect_named(res, c("id", "name", "value"))
  expect_equal(nrow(res), 4)  # 2 genes x 2 columns
  expect_setequal(unique(res$name), c("expr1", "expr2"))
})

test_that("funky_data() uses rownames as id and reverses factor levels", {
  df <- data.frame(x = 1:2, y = 3:4)
  rownames(df) <- c("g1", "g2")
  res <- funky_data(df, cols = 2)
  expect_true("id" %in% colnames(res))
  # factor levels should be reversed from original order
  expect_equal(levels(res$id), c("g2", "g1"))
})

test_that("funky_data() preserves cols2 columns (position relative to post-funky_id data)", {
  # After funky_id: columns are id, gene, expr, group (1,2,3,4)
  # cols=3 means expr is the data column; cols2=4 means keep group
  df <- data.frame(
    gene = c("A", "B"),
    expr = c(1, 2),
    group = c("case", "ctrl")
  )
  res <- funky_data(df, cols = 3, cols2 = 4)
  expect_true("group" %in% colnames(res))
  expect_equal(res$group, c("case", "ctrl"))
})

test_that("funky_data() sets id factor levels in reverse order of data rows", {
  df <- data.frame(
    gene = c("BRCA1", "TP53", "MYC"),
    expr = c(1, 2, 3)
  )
  rownames(df) <- c("BRCA1", "TP53", "MYC")
  res <- funky_data(df, cols = 2)
  expect_s3_class(res$id, "factor")
  expect_equal(levels(res$id), c("MYC", "TP53", "BRCA1"))
})

# ---- funky_fill_label ----

test_that("funky_fill_label() returns non-NULL for single col", {
  df <- data.frame(gene = 1:3, pval = c(0.01, 0.02, 0.03))
  ll <- funky_fill_label(df, cols = 2)
  expect_false(is.null(ll))
})

test_that("funky_fill_label() returns NULL for multiple cols", {
  df <- data.frame(gene = 1:3, a = 1:3, b = 4:6)
  expect_null(funky_fill_label(df, cols = c(2, 3)))
})

# ---- funky_theme ----

test_that("funky_theme() returns a ggplot2 theme object", {
  th <- funky_theme()
  expect_s3_class(th, "theme")
  expect_s3_class(th, "gg")
})

test_that("funky_theme() blanks axis.ticks.y and axis.text.y", {
  th <- funky_theme()
  expect_s3_class(th$axis.ticks.y, "element_blank")
  expect_s3_class(th$axis.text.y, "element_blank")
})

# ---- funky_setting ----

test_that("funky_setting() applies theme_blinds to each element of gglist", {
  suppressPackageStartupMessages(library(ggplot2))
  p1 <- ggplot(mtcars, aes(wt, mpg)) + geom_point()
  p2 <- ggplot(mtcars, aes(hp, mpg)) + geom_point()
  gglist <- list(p1, p2)

  res <- funky_setting(gglist, options = NULL)
  expect_length(res, 2)
  expect_s3_class(res[[1]], "ggplot")
  expect_s3_class(res[[2]], "ggplot")
})

test_that("funky_setting() applies extra options when provided", {
  suppressPackageStartupMessages(library(ggplot2))
  p1 <- ggplot(mtcars, aes(wt, mpg)) + geom_point()
  res <- funky_setting(list(p1), options = theme(plot.title = element_text(size = 20)))
  expect_s3_class(res[[1]], "ggplot")
})

## Aims
#
# 1. Reproduce upsetplot
# 2. Expand the function of upsetplot
#   2.1 Allow non-unique intersections
#   2.2 More applicable subplot types (boxplot, points), useful to show subsets properties


## Structures
#
# 1. plot and subplots
# 2. themes
# 3. data processing



## (PART) Plot

#' Plot a upset plot
#' 
#' This function generate a upset plot by creating a composite plot which contains subplots generated by ggplot2.
#' 
#' @title upset_plot
#' @param list a list of sets
#' @param nintersects number of intersects. If NULL, all intersections will show.
#' @param order.intersect.by one of 'size' or 'name'
#' @param order.set.by one of 'size' or 'name'
#' @param color.intersect.by color scheme for 'intersect' bars (e.g., "Set2"), default is "none"
#' @param color.set.by color scheme for 'set' bars (e.g., "Set3"), default is "none"
#' @param remove_empty_intersects remove the intersects which have zero elements. Default is TRUE.
#' @return an upset plot
#' @importFrom stats setNames
#' @export
#' 
#' @examples
#'  list = list(A = sample(LETTERS, 20),
#'              B = sample(LETTERS, 22),
#'              C = sample(LETTERS, 14),
#'              D = sample(LETTERS, 30, replace = TRUE))
#'  upset_plot(list)
#'  upset_plot(list, remove_empty_intersects = TRUE)
#'  upset_plot(list, order.intersect.by = "name")
#'  upset_plot(list, order.set.by = "name")
#'  upset_plot(list, nintersects = 6)
upset_plot = function(list,    # use 'a_' prefix for all `aplot` objects
                      nintersects = NULL,
                      order.intersect.by = c("size", "name"),
                      order.set.by = c("size", "name"),
                      color.intersect.by = "none",
                      color.set.by = "none",
                      remove_empty_intersects = TRUE){
  
  # process arguments
  if (is.null(names(list))){  # we need a named list
    names(list) = paste("Set", seq_along(list), sep = "_")
  }
  order.intersect.by = match.arg(order.intersect.by)
  order.set.by = match.arg(order.set.by)
  
  # subplot main
  data = tidy_main_subsets(list,
                           nintersects = nintersects,
                           order.intersect.by = order.intersect.by,
                           order.set.by = order.set.by,
                           remove_empty_intersects = remove_empty_intersects)
  
  id_levels <- levels(data$main_data$id)
  set_levels <- levels(data$left_data$set)
  
  # intersection color
  if (identical(color.intersect.by, "none")) {
    color_by_intersect <- setNames(rep("grey30", length(id_levels)), id_levels)
  } else {
    max_intersect_colors <- RColorBrewer::brewer.pal.info[color.intersect.by, "maxcolors"]
    base_colors <- RColorBrewer::brewer.pal(max_intersect_colors, color.intersect.by)
    color_vec <- grDevices::colorRampPalette(base_colors)(length(id_levels))
    color_by_intersect <- setNames(color_vec, id_levels)
  }
  
  # set color
  if (identical(color.set.by, "none")) {
    color_by_set <- setNames(rep("grey30", length(set_levels)), set_levels)
  } else {
    max_set_colors <- RColorBrewer::brewer.pal.info[color.set.by, "maxcolors"]
    base_set_colors <- RColorBrewer::brewer.pal(max_set_colors, color.set.by)
    color_vec_set <- grDevices::colorRampPalette(base_set_colors)(length(set_levels))
    color_by_set <- setNames(color_vec_set, set_levels)
  }
  
  # main plot
  p_main = upsetplot_main(data$main_data, color_by_id = color_by_intersect)
  
  # subplot top
  p_top = upsetplot_top(data$top_data, color_by_id = color_by_intersect)
  
  # subplot left
  p_left = upsetplot_left(data$left_data, color_by_set = color_by_set)
  
  # combine into a plot
  pp = aplot::insert_top(p_main, p_top, height = 4) |>
    aplot::insert_left(p_left, width=.2)
  class(pp) <- c("a_upset_plot", class(pp))
  
  return(pp)
}

upsetplot_main <- function(data, color_by_id = NULL) {
  if (!is.null(color_by_id)) {
    data$color <- color_by_id[as.character(data$id)]
  } 
  ggplot2::ggplot(data, aes(.data$id, .data$set)) +
    ggplot2::geom_point(aes(color = .data$color), size = 4, na.rm = FALSE) +
    ggplot2::geom_path(aes(group = .data$id, color = .data$color), linewidth = 1.5, na.rm = FALSE) +
    ggplot2::scale_color_identity() +
    ggplot2::labs(x = "Set Intersection", y = "") +
    theme_upset_main()
}

upsetplot_top <- function(data, color_by_id = NULL) {
  if (!is.null(color_by_id)) {
    data$fill <- color_by_id[as.character(data$id)]
  } 
  ggplot2::ggplot(data, aes(.data$id, .data$size)) +
    ggplot2::geom_col(aes(fill = .data$fill)) +
    ggplot2::geom_text(
      aes(label = ifelse(.data$size > 0, .data$size, NA)),
      vjust = -0.25,
      size = 4
    ) +
    ggplot2::scale_fill_identity() +
    ggplot2::labs(x = "", y = "Intersection Size") +
    theme_upset_top()
}


upsetplot_left <- function(data, color_by_set = NULL) {
  if (!is.null(color_by_set)) {
    data$fill <- color_by_set[as.character(data$set)]
  } 
  ggplot2::ggplot(data, aes(x = .data$size, y = .data$set)) +
    ggplot2::geom_col(aes(fill = .data$fill), orientation = "y") +
    ggplot2::scale_fill_identity() +
    ggplot2::scale_y_discrete(position = "right") +
    ggplot2::scale_x_reverse() +
    ggplot2::labs(x = "Set Size") +
    theme_upset_left()
}


## (PART) Theme

theme_upset_main = function(){
  ggplot2::theme_bw() +
    ggplot2::theme(
      axis.title.y = element_blank(),
      axis.ticks.x.bottom = element_blank(),
      axis.text.x.bottom = element_blank(),
      # panel.border = element_blank(),
      plot.margin = margin(t = -20)
    )
}

theme_upset_top = function(){
  ggplot2::theme_bw() +
    ggplot2::theme(
      axis.ticks.x.bottom = element_blank(),
      axis.text.x.bottom = element_blank(),
      # panel.border = element_blank(),
      plot.margin = margin(b = -20, unit = "pt")
    )
}

theme_upset_left = function(){
  ggplot2::theme_bw() +
    ggplot2::theme(
      axis.ticks.y = ggplot2::element_blank(),
      axis.title.y = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_blank(),
      panel.border = ggplot2::element_blank(),
      panel.grid.major = ggplot2::element_blank(),
      plot.margin = margin(r = -20)
    )
}


## (PART) retrieve tidy data from primary subset datasets

##' @importFrom forcats as_factor
tidy_main_subsets = function(list,
                             nintersects,
                             order.intersect.by,
                             order.set.by,
                             remove_empty_intersects = TRUE){
  
  data = get_all_subsets(list)
  set_name = names(list)
  
  # top data
  top_data = data |>
    dplyr::select(c('id','name', 'item','size')) |>
    dplyr::mutate(id = forcats::fct_reorder(.data$id, .data[[order.intersect.by]], .desc = TRUE))
  
  # left data
  left_data = dplyr::tibble(set = set_name,
                            name = set_name,
                            size = sapply(list, length)) |>
    dplyr::mutate(set = forcats::fct_reorder(.data$set, .data[[order.set.by]], .desc = TRUE))
  
  # main data
  main_data = data |>
    dplyr::select(c("id")) |>
    dplyr::mutate(set = .data$id) |>
    tidyr::separate_longer_delim("set", delim = "/")
  
  main_data$id <- factor(main_data$id, levels = levels(top_data$id))
  main_data$set = factor(set_name[as.integer(main_data$set)],
                         levels = levels(left_data$set))
  # filter intersections
  if (!is.null(nintersects)){
    keep_id = utils::head(levels(top_data$id), nintersects)
    main_data = main_data |> dplyr::filter(.data$id %in% keep_id)
    top_data = top_data |> dplyr::filter(.data$id %in% keep_id)
  }
  
  # remove empty intersects if necessary
  if (remove_empty_intersects) {
    non_empty_ids <- top_data$id[top_data$size > 0]
    top_data <- top_data[top_data$id %in% non_empty_ids, , drop = FALSE]
    main_data <- main_data[main_data$id %in% non_empty_ids, , drop = FALSE]
  }
  
  # return result as a list
  ret = list(top_data = top_data,
             left_data = left_data,
             main_data = main_data)
  return(ret)
}

## (PART) build primary data of subsets

#' Get the items/names/ids of subsets from a named list
#'
#' @param list a named list
#' @param name_separator default is /
#'
#' @return a tibble
#' @export
#'
#' @examples
#' list = list(A = sample(LETTERS, 20),
#'             B = sample(LETTERS, 22),
#'             C = sample(LETTERS, 24),
#'             D = sample(LETTERS, 30, replace = TRUE))
#' get_all_subsets(list)
get_all_subsets = function(list, name_separator = "/"){
  df = dplyr::tibble(
    id = get_all_subsets_ids(list, sep = name_separator),
    name = get_all_subsets_names(list, sep = name_separator),
    item = get_all_subsets_items(list)
  )
  df$size = sapply(df$item, length)
  return(df)
}

get_all_subsets_items <- function(list){
  n = length(list)
  c = combinations(n)
  lapply(c, function(i) overlap(list,i))
}

get_all_subsets_names <- function(list, sep = "/"){
  n = length(list)
  set_name = names(list)
  c = combinations(n)
  sapply(c, function(i) paste0(set_name[i], collapse = sep))
}

##' @importFrom yulab.utils combinations
get_all_subsets_ids <- function(list, sep = "/"){
  n <- length(list)
  c <- combinations(n)
  sapply(c, function(i) paste0(i, collapse = sep))
}

## (PART) subset calculations: the basics

##' @importFrom purrr reduce
overlap <- function(list, idx){
  slice1 <- list[idx]
  slice2 <- list[-idx]
  
  if (length(slice1) > 1L){
    overlap = slice1 |> purrr::reduce(intersect)
  } else if (length(slice1) == 1L){
    overlap <- unlist(slice1)
  } else {
    overlap <- NULL
  }
  
  if (length(slice2) > 1L){
    outmember <- slice2 |> purrr::reduce(union)
  } else if (length(slice2) == 1L){
    outmember <- unlist(slice2)
  } else {
    outmember <- NULL
  }
  
  setdiff(overlap, outmember)
}

# Tests for upset.R data functions -------------------------------------------

test_that("overlap() computes setdiff of intersect(idx) vs union(-idx)", {
  lst <- list(
    A = c("a", "b", "c", "d"),
    B = c("b", "c", "d", "e"),
    C = c("c", "d", "e", "f")
  )

  # A only (intersect of A, minus union of B,C)
  expect_setequal(overlap(lst, 1), "a")
  # B only
  expect_setequal(overlap(lst, 2), character(0))
  # C only
  expect_setequal(overlap(lst, 3), "f")
  # A & B intersection
  expect_setequal(overlap(lst, c(1, 2)), c("b"))
  # A & C intersection
  expect_setequal(overlap(lst, c(1, 3)), character(0))
  # B & C intersection
  expect_setequal(overlap(lst, c(2, 3)), "e")
  # A & B & C intersection
  expect_setequal(overlap(lst, c(1, 2, 3)), c("c", "d"))
})

test_that("overlap() handles edge cases", {
  # Single set
  lst1 <- list(A = letters[1:5])
  expect_setequal(overlap(lst1, 1), letters[1:5])

  # Empty set
  lst2 <- list(A = character(0), B = letters[1:3])
  expect_setequal(overlap(lst2, 1), character(0))
  expect_setequal(overlap(lst2, 2), letters[1:3])

  # Fully overlapping sets
  lst3 <- list(A = 1:3, B = 1:3)
  expect_setequal(overlap(lst3, 1), character(0))
  expect_setequal(overlap(lst3, 2), character(0))
  expect_setequal(overlap(lst3, c(1, 2)), as.character(1:3))
})

test_that("get_all_subsets_ids() returns slash-separated indices", {
  lst <- list(A = 1:3, B = 2:4, C = 3:5)
  ids <- get_all_subsets_ids(lst)
  expect_equal(ids[1], "1")
  expect_equal(ids[2], "2")
  expect_equal(ids[3], "3")
  expect_equal(ids[4], "1/2")
  expect_equal(ids[7], "1/2/3")
  expect_length(ids, 7)  # 2^3 - 1
})

test_that("get_all_subsets_names() returns slash-separated set names", {
  lst <- list(X = 1:3, Y = 2:4)
  nms <- get_all_subsets_names(lst)
  expect_equal(nms, c("X", "Y", "X/Y"))
})

test_that("get_all_subsets_items() returns list of character vectors", {
  lst <- list(A = c("a", "b", "c"), B = c("b", "c", "d"))
  items <- get_all_subsets_items(lst)
  expect_type(items, "list")
  expect_length(items, 3)  # 2^2 - 1
  expect_setequal(items[[1]], "a")
  expect_setequal(items[[2]], "d")
  expect_setequal(items[[3]], c("b", "c"))
})

test_that("get_all_subsets() returns a tibble with id/name/item/size columns", {
  # A: 1,2,3,4,5; B: 3,4,5,6,7
  # A-only = {1,2} = 2; B-only = {6,7} = 2; A&B = {3,4,5} = 3
  lst <- list(A = 1:5, B = 3:7)
  res <- get_all_subsets(lst)
  expect_s3_class(res, "tbl_df")
  expect_named(res, c("id", "name", "item", "size"))
  expect_equal(res$name, c("A", "B", "A/B"))
  expect_equal(res$id, c("1", "2", "1/2"))
  expect_equal(res$size, c(2, 2, 3))
})

test_that("tidy_main_subsets() returns list of main/top/left data", {
  lst <- list(A = 1:5, B = 3:7)
  res <- tidy_main_subsets(lst,
    nintersects = NULL,
    order.intersect.by = "size",
    order.set.by = "size",
    remove_empty_intersects = TRUE
  )
  expect_type(res, "list")
  expect_named(res, c("top_data", "left_data", "main_data"))

  # left_data: one row per set, with size
  expect_equal(nrow(res$left_data), 2)
  expect_equal(unname(res$left_data$size), c(5, 5))
  expect_setequal(as.character(res$left_data$set), c("A", "B"))

  # top_data: one row per intersection
  expect_equal(nrow(res$top_data), 3)  # A-only, B-only, A&B
  expect_setequal(res$top_data$size, c(2, 2, 3))

  # main_data: expanded id -> set mapping
  main_ids <- unique(as.character(res$main_data$id))
  expect_length(main_ids, 3)
})

test_that("tidy_main_subsets() respects nintersects", {
  lst <- list(A = 1:5, B = 3:7)
  res <- tidy_main_subsets(lst, nintersects = 2,
    order.intersect.by = "size",
    order.set.by = "size"
  )
  expect_equal(nrow(res$top_data), 2)
})

test_that("tidy_main_subsets() accepts order.intersect.by = 'name'", {
  lst <- list(A = 1:5, B = 3:7)
  res <- tidy_main_subsets(lst,
    nintersects = NULL,
    order.intersect.by = "name",
    order.set.by = "size"
  )
  # ordering by name with fct_reorder (non-numeric key) — verifies no error
  expect_equal(nrow(res$top_data), 3)
  expect_setequal(as.character(res$top_data$name), c("A", "B", "A/B"))
})

test_that("tidy_main_subsets() handles remove_empty_intersects = FALSE", {
  lst <- list(A = 1:5, B = 3:7)
  res <- tidy_main_subsets(lst,
    nintersects = NULL,
    order.intersect.by = "size",
    order.set.by = "size",
    remove_empty_intersects = FALSE
  )
  expect_equal(nrow(res$top_data), 3)  # no empty to remove in this case
})

#' ploting oncoplot with aplot
#' @param maf MAF object. 
#' @param genes the gene names or the number, default is 20.
#' @return \code{oncoplot} object, which is also a \code{aplot} object
#' @importFrom aplot insert_top insert_right insert_bottom
#' @export
#' @examples
#' \donttest{
#' laml.maf <- system.file("extdata", "tcga_laml.maf.gz", package = "maftools")
#' laml.clin <- system.file('extdata', 'tcga_laml_annot.tsv', package = 'maftools')
#' laml <- maftools::read.maf(maf = laml.maf, clinicalData = laml.clin)
#' oncoplot(maf = laml, genes = 20)
#' }
oncoplot <- function(maf, genes = 20) {
    yulab.utils::check_pkg("maftools")

    comp <- oncoplot_components(maf, genes)
    p_spacer <- ggplot() + ggfun::theme_transparent()

    pp <- comp$main |>
        insert_top(comp$sample, height=.2) |>
        insert_right(p_spacer, width=.01) |>
        insert_right(comp$gene, width=.2) |>
        insert_bottom(p_spacer, height=.1)

    class(pp) <- c("oncoplot", class(pp))

    return(pp)
}

#' Get reusable oncoplot components
#'
#' Build the reusable panels and tidy data used by [oncoplot()].
#' This function is intended for downstream packages that want to assemble
#' an oncoplot with additional tracks while reusing the aplotExtra mutation
#' matrix, sample burden, gene burden, and mutation color mapping.
#'
#' @param maf A MAF object.
#' @param genes The gene names or number of top mutated genes.
#'
#' @return A list with components:
#' \describe{
#'   \item{data}{Tidy oncoplot matrix.}
#'   \item{main}{Main mutation tile plot.}
#'   \item{sample}{Sample-level top annotation plot.}
#'   \item{gene}{Gene-level side annotation plot.}
#'   \item{colors}{Named mutation-class color vector.}
#' }
#' @export
oncoplot_components <- function(maf, genes = 20) {
    d <- oncoplot_tidy_onco_matrix(maf, genes)

    structure(
        list(
            data = d,
            main = oncoplot_main(maf, genes),
            sample = oncoplot_sample(maf, genes),
            gene = oncoplot_gene(maf, genes, ylab = "percentage"),
            colors = get_vcColors(websafe = FALSE)
        ),
        class = "aplotextra_oncoplot_components"
    )
}

#' @importFrom ggplot2 geom_tile
#' @importFrom ggplot2 annotation_custom
#' @importFrom ggplot2 coord_cartesian
oncoplot_main <- function(maf, genes = 20) {
    d <- oncoplot_tidy_onco_matrix(maf, genes)

    p <- ggplot(d, aes(x=.data$Sample, y=.data$Gene, fill=.data$Type)) +
        geom_tile(colour="white", linewidth=.01) + 
        oncoplot_setting(continuous = FALSE) +
        theme(legend.position = "bottom", 
            axis.text.y.left=element_text(face='italic')) 

    leg <- ggfun::get_legend(p)
    p + theme(legend.position = "none", 
            plot.margin = margin(b = 60, r = 5)) +
        annotation_custom(leg, ymin=-1.5, ymax=-.5) + 
        coord_cartesian(clip='off')
    
}

##' @importFrom aplot ylab2
oncoplot_sample <- function(maf, genes = 20, sort = FALSE) {    
    numMat <- get_oncoplot_numericMatrix(maf, genes)
    samp_sum <- obtain.sample.summary.MAF(x = maf) |>
        as.data.frame() |>
        dplyr::select(!.data$total) 

    i <- match(colnames(numMat), samp_sum$Tumor_Sample_Barcode)
    samp_sum <- samp_sum[i, , drop=FALSE]
    d <- tidyr::pivot_longer(samp_sum, -1) |>
        dplyr::rename(Sample=.data$Tumor_Sample_Barcode, Type=.data$name, Freq=.data$value)
    d$Sample <- factor(d$Sample, levels = unique(colnames(numMat)))
    if (sort) {
        td <- dplyr::group_by(.data$d, .data$Type) |> 
            dplyr::summarize(total = sum(.data$Freq)) |> 
            dplyr::arrange(.data$total) |> 
            dplyr::pull(.data$Type)
        d$Type <- factor(d$Type, levels = td)
    }

    p <- ggplot(d, aes(x=.data$Sample,y=.data$Freq,fill=.data$Type)) +
        geom_col(position="stack") +
        oncoplot_setting() +
        ylab2("TMB") 
    
    return(p)
}




#' @importFrom rlang .data
#' @importFrom ggplot2 ggplot
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 coord_flip 
#' @importFrom ggplot2 geom_bar
#' @importFrom ggplot2 xlab
#' @importFrom ggplot2 scale_x_discrete
#' @importFrom ggplot2 scale_y_discrete
#' @importFrom ggplot2 margin
#' @importFrom grid textGrob
#' @importFrom grid gpar
#' @importFrom aplot xlab2
oncoplot_gene <- function(maf, genes = 20, ylab = 'gene') {
    ylab <- match.arg(ylab, c("gene", "percentage"))

    d <- oncoplot_tidy_onco_matrix(maf, genes)
    d <- d[!is.na(d$Type), ]

    p <- ggplot(d, aes(y = .data$Gene, fill = .data$Type)) + 
        geom_bar(position='stack', orientation = 'y') + 
        oncoplot_setting(noxaxis = FALSE, scale='none') #+
        # xlab("No. of samples") #+ 
        # guides(y= guide_axis_label_trans(~str_pad(rev(percent_alt), 5)))

    if (ylab == 'percentage') {
        numMat <- get_oncoplot_numericMatrix(maf, genes)
        totSamps = as.numeric(maf@summary[3, 'summary'])
        percent <- apply(numMat, 1, function(x) sum(x >0))/totSamps
        percent_alt <- paste0(round(percent * 100), '%')
        p <- p + scale_y_discrete(breaks = rownames(numMat),
                                labels = percent_alt,
                                expand = c(0, 0))
    }

    p <- p + xlab2("No. of samples")

    return(p)
}

#' @importFrom ggplot2 theme_minimal
oncoplot_scale <- function(continuous = TRUE, scale = 'y') {
    scale <- match.arg(scale, c('x', 'y', 'none'))
    if (scale == 'none') {
        return(NULL)
    } 
    
    if (scale == 'x') {
        if (continuous) {
            scale_fun <- ggplot2::scale_x_continuous
        } else {
            scale_fun <- ggplot2::scale_x_discrete
        }
    } else {
        if (continuous) {
            scale_fun <- ggplot2::scale_y_continuous
        } else{
            scale_fun <- ggplot2::scale_y_discrete
        }
    }
    
    scale_fun(expand = c(0, 0))

}

#' @importFrom ggplot2 theme_minimal
#' @importFrom ggplot2 theme
#' @importFrom ggplot2 ylab
oncoplot_setting <- function(noxaxis = TRUE, continuous = TRUE, scale='y') {  
    list(
        theme_minimal(),
        if (noxaxis) ggfun::theme_noxaxis(),
        theme(legend.position = "none", panel.grid.major = element_blank()),
        oncoplot_scale(continuous = continuous, scale = scale),
        oncoplot_fill(),
        xlab(NULL),
        ylab(NULL)
    )
}

#' @importFrom ggplot2 scale_fill_manual
oncoplot_fill <- function(breaks=NULL, values=NULL, name = NULL, na.value = "#bdbdbd") {
    vc_col <- get_vcColors(websafe = FALSE)  # maftools color setting
    if (is.null(values)) {
        values <- vc_col
    } 
    if (is.null(breaks)) {
        vc_lev <- names(vc_col)
        breaks <- factor(vc_lev, levels = rev(vc_lev))
    }
    scale_fill_manual(name = NULL, 
            breaks = breaks, 
            values = values, 
            na.value = na.value)
}


get_oncoplot_genes <- function(maf, genes = 20) {
    if (is.numeric(genes)) {
        if (length(genes) == 1) {
            # Top N genes
            genes <- obtain.gene.summary.MAF(x = maf)[1:genes, "Hugo_Symbol"]  
        } else {
            # as index
            genes <- obtain.gene.summary.MAF(x = maf)[genes, "Hugo_Symbol"]
        }
    }
    # if character, return as it is.

    return(genes)
}

get_oncoplot_numericMatrix <- function(maf, genes = 20) {
    genes <- get_oncoplot_genes(maf, genes = genes)
    om <- createOncoMatrix(m = maf, g = genes)
    numMat = om$numericMatrix  # gene/sample ~ frequency
    return(numMat)
}


oncoplot_tidy_onco_matrix <- function(maf, genes = 20) {
    genes <- get_oncoplot_genes(maf, genes)
    om <- createOncoMatrix(m = maf, g = genes)
    mat_origin = om$oncoMatrix # gene/sample ~ variant type

    d = mat_origin[rev(rownames(mat_origin)),] |>
        as.data.frame() |>
        tibble::rownames_to_column('Gene') |>
        tidyr::pivot_longer(-1) |>
        dplyr::rename(Sample = "name", Type = "value")

    d$Type[d$Type == ""] <- NA #"Non_Mut"
    d$Gene <- factor(d$Gene, levels = rev(rownames(mat_origin)))
    d$Sample <- factor(d$Sample, levels = colnames(mat_origin))
    return(d)
}

obtain.sample.summary.MAF <- function(x){
    x@variant.classification.summary
}

obtain.gene.summary.MAF <- function(x){
    x@gene.summary
}

# To add annotation on right y axis: https://github.com/tidyverse/ggplot2/issues/3171
# guide_axis_label_trans <- function(label_trans = identity, ...) {
#   axis_guide <- guide_axis(...)
#   axis_guide$label_trans <- rlang::as_function(label_trans)
#   class(axis_guide) <- c("guide_axis_trans", class(axis_guide))
#   axis_guide
# }

# guide_train.guide_axis_trans <- function(x, ...) {
#   trained <- NextMethod()
#   trained$key$.label <- x$label_trans(trained$key$.label)
#   trained
# }

#' @import maftools
#' @importFrom yulab.utils get_fun_from_pkg
get_vcColors <- yulab.utils::get_fun_from_pkg('maftools', 'get_vcColors')
createOncoMatrix <- yulab.utils::get_fun_from_pkg('maftools', 'createOncoMatrix')

if(getRversion() >= "2.15.1")  {
    utils::globalVariables(c(".", "Hugo_Symbol", 
       "Tumor_Sample_Barcode", "Variant_Classification","Variant_Classification_temp","Variant_Type")
    )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upset.R
\name{upset_plot}
\alias{upset_plot}
\title{upset_plot}
\usage{
upset_plot(
  list,
  nintersects = NULL,
  order.intersect.by = c("size", "name"),
  order.set.by = c("size", "name"),
  color.intersect.by = "none",
  color.set.by = "none",
  remove_empty_intersects = TRUE
)
}
\arguments{
\item{list}{a list of sets}

\item{nintersects}{number of intersects. If NULL, all intersections will show.}

\item{order.intersect.by}{one of 'size' or 'name'}

\item{order.set.by}{one of 'size' or 'name'}

\item{color.intersect.by}{color scheme for 'intersect' bars (e.g., "Set2"), default is "none"}

\item{color.set.by}{color scheme for 'set' bars (e.g., "Set3"), default is "none"}

\item{remove_empty_intersects}{remove the intersects which have zero elements. Default is TRUE.}
}
\value{
an upset plot
}
\description{
Plot a upset plot
}
\details{
This function generate a upset plot by creating a composite plot which contains subplots generated by ggplot2.
}
\examples{
 list = list(A = sample(LETTERS, 20),
             B = sample(LETTERS, 22),
             C = sample(LETTERS, 14),
             D = sample(LETTERS, 30, replace = TRUE))
 upset_plot(list)
 upset_plot(list, remove_empty_intersects = TRUE)
 upset_plot(list, order.intersect.by = "name")
 upset_plot(list, order.set.by = "name")
 upset_plot(list, nintersects = 6)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upset.R
\name{get_all_subsets}
\alias{get_all_subsets}
\title{Get the items/names/ids of subsets from a named list}
\usage{
get_all_subsets(list, name_separator = "/")
}
\arguments{
\item{list}{a named list}

\item{name_separator}{default is /}
}
\value{
a tibble
}
\description{
Get the items/names/ids of subsets from a named list
}
\examples{
list = list(A = sample(LETTERS, 20),
            B = sample(LETTERS, 22),
            C = sample(LETTERS, 24),
            D = sample(LETTERS, 30, replace = TRUE))
get_all_subsets(list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funkyheatmap.R
\name{funky_point}
\alias{funky_point}
\title{funky_point}
\usage{
funky_point(data, cols, cols2 = NULL, ...)
}
\arguments{
\item{data}{data frame}

\item{cols}{selected columns}

\item{cols2}{selected columns to keep names}

\item{...}{additional parameters, passing to \code{\link[ggstar]{geom_star}}}
}
\value{
ggplot object
}
\description{
create dot plot for funkyheatmap
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funkyheatmap.R
\name{funky_heatmap}
\alias{funky_heatmap}
\title{funky_heatmap}
\usage{
funky_heatmap(..., data = NULL, widths = NULL, options = NULL)
}
\arguments{
\item{...}{funky plots (e.g., outputs of \code{funky_point}, \code{funky_bar}, etc.)}

\item{data}{If data is provided, create a funkyheatmap from it. Otherwise, create composite plot from \code{...}}

\item{widths}{relative widths of the plots}

\item{options}{any ggplot component that can be added to the plots}
}
\value{
gglist object
}
\description{
create a funkyheatmap
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oncoplot.r
\name{oncoplot}
\alias{oncoplot}
\title{ploting oncoplot with aplot}
\usage{
oncoplot(maf, genes = 20)
}
\arguments{
\item{maf}{MAF object.}

\item{genes}{the gene names or the number, default is 20.}
}
\value{
\code{oncoplot} object, which is also a \code{aplot} object
}
\description{
ploting oncoplot with aplot
}
\examples{
\donttest{
laml.maf <- system.file("extdata", "tcga_laml.maf.gz", package = "maftools")
laml.clin <- system.file('extdata', 'tcga_laml_annot.tsv', package = 'maftools')
laml <- maftools::read.maf(maf = laml.maf, clinicalData = laml.clin)
oncoplot(maf = laml, genes = 20)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funkyheatmap.R
\name{funky_text}
\alias{funky_text}
\title{funky_text}
\usage{
funky_text(data, cols = 1, hjust = 0)
}
\arguments{
\item{data}{data frame}

\item{cols}{selected column}

\item{hjust}{text alignment adjustment}
}
\value{
ggplot object
}
\description{
create text plot (i.e., rownames) for funkyheatmap
}
\author{
Guangchuang Yu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aplotExtra-package.R
\docType{package}
\name{aplotExtra-package}
\alias{aplotExtra}
\alias{aplotExtra-package}
\title{aplotExtra: Creating Composite Plots using 'aplot'}
\description{
Many complex plots are actually composite plots, such as 'oncoplot', 'funkyheatmap', 'upsetplot', etc. We can produce subplots using 'ggplot2' and combine them to create composite plots using 'aplot'. In this way, it is easy to customize these complex plots, by adding, deleting or modifying subplots in the final plot. This package provides a set of utilities to help users to create subplots and complex plots.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/YuLab-SMU/aplotExtra}
}

}
\author{
\strong{Maintainer}: Guangchuang Yu \email{guangchuangyu@gmail.com} (\href{https://orcid.org/0000-0002-6485-8781}{ORCID})

Authors:
\itemize{
  \item Guangchuang Yu \email{guangchuangyu@gmail.com} (\href{https://orcid.org/0000-0002-6485-8781}{ORCID})
}

Other contributors:
\itemize{
  \item Shuangbin Xu \email{xshuangbin@163.com} (\href{https://orcid.org/0000-0003-3513-5362}{ORCID}) [contributor]
  \item Chun-Hui Gao \email{gaospecial@gmail.com} (\href{https://orcid.org/0000-0002-1445-7939}{ORCID}) [contributor]
  \item Shensuo Li \email{lishensuo@163.com} (\href{https://orcid.org/0009-0006-3229-3700}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oncoplot.r
\name{oncoplot_components}
\alias{oncoplot_components}
\title{Get reusable oncoplot components}
\usage{
oncoplot_components(maf, genes = 20)
}
\arguments{
\item{maf}{A MAF object.}

\item{genes}{The gene names or number of top mutated genes.}
}
\value{
A list with components:
\describe{
\item{data}{Tidy oncoplot matrix.}
\item{main}{Main mutation tile plot.}
\item{sample}{Sample-level top annotation plot.}
\item{gene}{Gene-level side annotation plot.}
\item{colors}{Named mutation-class color vector.}
}
}
\description{
Build the reusable panels and tidy data used by \code{\link[=oncoplot]{oncoplot()}}.
This function is intended for downstream packages that want to assemble
an oncoplot with additional tracks while reusing the aplotExtra mutation
matrix, sample burden, gene burden, and mutation color mapping.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funkyheatmap.R
\name{funky_bar}
\alias{funky_bar}
\title{funky_bar}
\usage{
funky_bar(data, cols)
}
\arguments{
\item{data}{data frame}

\item{cols}{selected columns}
}
\value{
ggplot object
}
\description{
create bar plot for funkyheatmap
}
\author{
Guangchuang Yu
}
